"""This data helps download the movisens data"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_Config.ipynb.

# %% auto 0
__all__ = ['Config']

# %% ../nbs/00_Config.ipynb 3
import yaml
from pathlib import Path
import sys

# %% ../nbs/00_Config.ipynb 4
class Config:
    def __init__(self, config_file='config.yaml'):
        if 'ipykernel' in sys.modules:
            # Running in a Jupyter Notebook
            config_path = Path.cwd() / '..' / config_file
        else:
            # Running as a Python script
            config_path = Path(__file__).parent / config_file

        if not config_path.exists():
            raise FileNotFoundError(f"Configuration file not found at {config_path}")

        self.config = yaml.safe_load(config_path.read_text())
        self.project_root = config_path

    def get_path(self, key: str) -> Path:
        try:
            relative_path = self.config["paths"][key]
            return (self.project_root / relative_path).resolve()
        except KeyError:
            raise KeyError(f"Path '{key}' not found in configuration.")

    def get_setting(self, key: str):
        try:
            return self.config["settings"][key]
        except KeyError:
            raise KeyError(f"Setting '{key}' not found in configuration.")


